# Find the hiredis library.
#
# This module defines
#  HIREDIS_FOUND             - True if HIREDIS was found.
#  HIREDIS_INCLUDE_DIRS      - Include directories for HIREDIS headers.
#  HIREDIS_LIBRARIES         - Libraries for HIREDIS.
#
# To specify an additional directory to search, set HIREDIS_ROOT.
#
# Copyright (c) 2010, Ewen Cheslack-Postava
# Based on FindSQLite3.cmake by:
#  Copyright (c) 2006, Jaroslaw Staniek, <js@iidea.pl>
#  Extended by Siddhartha Chaudhuri, 2008.
#
# Redistribution and use is allowed according to the terms of the BSD license.
#

SET(HIREDIS_FOUND FALSE)
SET(HIREDIS_INCLUDE_DIRS)
SET(HIREDIS_LIBRARIES)

IF(HIREDIS_ROOT)
  SET(SEARCH_PATHS
    ${HIREDIS_ROOT}
    ${HIREDIS_ROOT}/include
    ${SEARCH_PATHS}
    )
ENDIF()

FIND_PATH(HIREDIS_INCLUDE_DIRS
  NAMES hiredis/hiredis.h
  PATHS ${SEARCH_PATHS}
  NO_DEFAULT_PATH)
IF(NOT HIREDIS_INCLUDE_DIRS)  # now look in system locations
  FIND_PATH(HIREDIS_INCLUDE_DIRS NAMES hiredis/hiredis.h)
ENDIF(NOT HIREDIS_INCLUDE_DIRS)

SET(HIREDIS_LIBRARY_DIRS)
IF(HIREDIS_ROOT)
  SET(HIREDIS_LIBRARY_DIRS ${HIREDIS_ROOT})
  IF(EXISTS "${HIREDIS_ROOT}/lib")
    SET(HIREDIS_LIBRARY_DIRS ${HIREDIS_LIBRARY_DIRS} ${HIREDIS_ROOT}/lib)
  ENDIF()
ENDIF()

# NVCORE
# Without system dirs
FIND_LIBRARY(HIREDIS_LIBRARY
  NAMES hiredis
  PATHS ${HIREDIS_LIBRARY_DIRS}
  NO_DEFAULT_PATH
  )
IF(NOT HIREDIS_LIBRARY)  # now look in system locations
  FIND_LIBRARY(HIREDIS_LIBRARY NAMES hiredis)
ENDIF(NOT HIREDIS_LIBRARY)

SET(HIREDIS_LIBRARIES)
IF(HIREDIS_LIBRARY)
  SET(HIREDIS_LIBRARIES ${HIREDIS_LIBRARY})
ENDIF()

IF(HIREDIS_INCLUDE_DIRS AND HIREDIS_LIBRARIES)
  SET(HIREDIS_FOUND TRUE)
  IF(NOT HIREDIS_FIND_QUIETLY)
    MESSAGE(STATUS "Found HIREDIS: headers at ${HIREDIS_INCLUDE_DIRS}, libraries at ${HIREDIS_LIBRARY_DIRS} :: ${HIREDIS_LIBRARIES}")
  ENDIF(NOT HIREDIS_FIND_QUIETLY)
ELSE(HIREDIS_INCLUDE_DIRS AND HIREDIS_LIBRARIES)
  SET(HIREDIS_FOUND FALSE)
  IF(HIREDIS_FIND_REQUIRED)
    MESSAGE(STATUS "HIREDIS not found")
  ENDIF(HIREDIS_FIND_REQUIRED)
ENDIF(HIREDIS_INCLUDE_DIRS AND HIREDIS_LIBRARIES)

MARK_AS_ADVANCED(HIREDIS_INCLUDE_DIRS HIREDIS_LIBRARIES)
